"""主程序：将 PDF 转换为 PNG 图片，然后逐张调用截图工具进行处理"""

import dis
import os
import time
import threading
import cv2
import shutil
import glob
import argparse
from notebooklm2ppt.cli import main

if __name__ == "__main__":
    main()



"""工具函数模块"""

from .image_viewer import show_image_fullscreen
from .image_inpainter import inpaint_image
from .screenshot_automation import take_fullscreen_snip, mouse, screen_height, screen_width

__all__ = [
    'show_image_fullscreen',
    'inpaint_image',
    'take_fullscreen_snip',
    'mouse',
    'screen_height',
    'screen_width',
]



import fitz  # PyMuPDF
import os
from pathlib import Path
from .utils.image_inpainter import inpaint_image

def pdf_to_png(pdf_path, output_dir=None, dpi=150, inpaint=False, pages=None):
    """
    将 PDF 文件转换为多个 PNG 图片
    
    参数:
        pdf_path: PDF 文件路径
        output_dir: 输出目录，默认为 PDF 同目录的 pdf_name_pngs 文件夹
        dpi: 分辨率，默认 150
    """
    # 打开 PDF 文件
    pdf_doc = fitz.open(pdf_path)
    
    # 确定输出目录
    if output_dir is None:
        pdf_name = Path(pdf_path).stem  # 获取 PDF 文件名（不含扩展名）
        output_dir = Path(pdf_path).parent / f"{pdf_name}_pngs"
    else:
        output_dir = Path(output_dir)
    
    # 创建输出目录
    output_dir.mkdir(parents=True, exist_ok=True)
    
    # 转换因子：DPI / 72（默认屏幕 DPI）
    zoom = dpi / 72
    mat = fitz.Matrix(zoom, zoom)
    
    # 遍历每一页
    page_count = len(pdf_doc)  # 在关闭文档前获取页数
    # 规范 pages 参数为集合，方便判断
    pages_set = None
    if pages is not None:
        pages_set = set(pages)

    png_names = []
    for page_num, page in enumerate(pdf_doc, 1):
        # 如果指定了 pages，则跳过不在范围内的页码
        if pages_set is not None and page_num not in pages_set:
            continue
        # 渲染页面为图片
        pix = page.get_pixmap(matrix=mat, alpha=False)
        
        # 保存为 PNG
        output_path = output_dir / f"page_{page_num:04d}.png"
        
        png_names.append(output_path.name)

        if os.path.exists(output_path):
            print(f"跳过已存在的文件: {output_path}")
            continue
        pix.save(output_path)
        print(f"✓ 已保存: {output_path}")
        if inpaint:
            inpaint_image(str(output_path), str(output_path))
            print(f"✓ 已修复: {output_path}")

        
            
    pdf_doc.close()
    print(f"\n完成! 共转换 {page_count} 页，输出目录: {output_dir}")
    return png_names

if __name__ == "__main__":
    # 使用示例
    pdf_file = "Hackathon_Architect_Playbook.pdf"  # 修改为你的 PDF 文件路径
    
    if os.path.exists(pdf_file):
        pdf_to_png(pdf_file, dpi=150)
    else:
        print(f"错误: 文件 {pdf_file} 不存在")



import toml
import os
import argparse

# pyinstaller --clean -F -w -n notebooklm2ppt_{version} --optimize=2 --collect-all spire.presentation main.py 

if __name__ == "__main__":
    # 读取toml中的版本号
    # 执行编译命令

    parser = argparse.ArgumentParser(description="Compile notebooklm2ppt into a standalone executable.")
    parser.add_argument(
        "--as_dir",
        action="store_true",
        help="Compile as a directory instead of a single file executable.",
    )
    args = parser.parse_args()

    
    with open("pyproject.toml", "r", encoding="utf-8") as f:
        pyproject_data = toml.load(f)


    version = pyproject_data["project"]["version"]
    output_name = f"notebooklm2ppt-{version}"
    print(f"编译版本: {output_name}")

    if not args.as_dir:
        os.system(f'pyinstaller --clean -F -w -n {output_name} --optimize=2 --collect-all spire.presentation main.py ')
    else:
        os.system(f'pyinstaller --clean -D -w -n {output_name} --optimize=2 --collect-all spire.presentation main.py ')


"""命令行界面：将 PDF 转换为可编辑 PowerPoint 演示文稿"""

import os
import time
import threading
import cv2
import shutil
import argparse
import sys
from pathlib import Path
from .pdf2png import pdf_to_png
from .utils.image_viewer import show_image_fullscreen
from .utils.screenshot_automation import take_fullscreen_snip, mouse, screen_height, screen_width


def process_pdf_to_ppt(pdf_path, png_dir, ppt_dir, delay_between_images=2, inpaint=True, dpi=150, timeout=50, display_height=None, 
                    display_width=None, done_button_offset=None, capture_done_offset: bool = True, pages=None, update_offset_callback=None):
    """
    将 PDF 转换为 PNG 图片，然后对每张图片进行截图处理
    
            try:
        pdf_path: PDF 文件路径
            # 对第一页允许用户手动点击并捕获完成按钮偏移（如果未保存且启用）
            capture_offset = (idx == 1 and capture_done_offset)
            success, ppt_filename, computed_offset = take_fullscreen_snip(
                check_ppt_window=True,
                ppt_check_timeout=timeout,
                width=display_width,
                height=display_height,
                pc_manager_version=pc_manager_version,
                done_button_right_offset=done_button_offset,
                capture_done_offset_if_missing=capture_offset,
                force_capture=force_calibrate,
            )
            # 如果本次截屏捕获并保存了偏移，回调通知（GUI 可使用此回调更新显示）
            try:
                if computed_offset is not None and offset_saved_callback:
                    offset_saved_callback(int(computed_offset))
            except Exception:
                pass
        pc_manager_version: 电脑管家版本号；3.19及以上自动使用 190，低于3.19 使用 210
        done_button_offset: 完成按钮右侧偏移量，传入数字时优先使用，不传则按版本推断
    """
    # 1. 将 PDF 转换为 PNG 图片
    print("=" * 60)
    print("步骤 1: 将 PDF 转换为 PNG 图片")
    print("=" * 60)
    
    if not os.path.exists(pdf_path):
        print(f"错误: PDF 文件 {pdf_path} 不存在")
        return
    
    png_names = pdf_to_png(pdf_path, png_dir, dpi=dpi, inpaint=inpaint, pages=pages)
    
    # 创建ppt输出目录
    ppt_dir.mkdir(exist_ok=True, parents=True)
    print(f"PPT输出目录: {ppt_dir}")
    
    # 获取用户的下载文件夹路径
    downloads_folder = Path.home() / "Downloads"
    print(f"下载文件夹: {downloads_folder}")
    
    # 2. 获取所有 PNG 图片文件并排序
    png_files = [png_dir / name for name in png_names]
    
    if not png_files:
        print(f"错误: 在 {png_dir} 中没有找到 PNG 图片")
        return
    
    print("\n" + "=" * 60)
    print(f"步骤 2: 处理 {len(png_files)} 张 PNG 图片")
    print("=" * 60)
    
    # 设置显示窗口尺寸（如果未指定则使用屏幕尺寸）
    if display_height is None:
        display_height = screen_height
    if display_width is None:
        display_width = screen_width
    
    print(f"显示窗口尺寸: {display_width} x {display_height}")

    
    # 3. 对每张图片进行截图处理
    for idx, png_file in enumerate(png_files, 1):
        print(f"\n[{idx}/{len(png_files)}] 处理图片: {png_file.name}")
        
        stop_event = threading.Event()
        
        def _viewer():
            """在线程中显示图片"""
            show_image_fullscreen(str(png_file), display_height=display_height)
            # 维持 OpenCV 事件循环
            while not stop_event.is_set():
                cv2.waitKey(50)
            # 关闭窗口
            try:
                cv2.destroyAllWindows()
            except Exception:
                pass
        
        # 启动图片显示线程
        viewer_thread = threading.Thread(
            target=_viewer, 
            name=f"opencv_viewer_{idx}", 
            daemon=True
        )
        viewer_thread.start()
        
        # 等待窗口稳定
        time.sleep(3)
        
        try:
            # 执行全屏截图并检测PPT窗口
            # 对第一页允许用户手动点击并捕获完成按钮偏移（如果未保存或被强制要求）
            capture_offset = (idx == 1 and capture_done_offset)
            if capture_offset:
                done_button_offset = None  # 强制重新捕获偏移
            else:
                assert done_button_offset is not None, "必须提供完成按钮偏移量"
            success, ppt_filename, computed_offset = take_fullscreen_snip(
                check_ppt_window=True,
                ppt_check_timeout=timeout,
                width=display_width,
                height=display_height,
                done_button_right_offset=done_button_offset,
            )
            if success and computed_offset is not None:
                print(f"捕获到的完成按钮偏移: {computed_offset}")
                done_button_offset = computed_offset  # 更新为最新捕获的偏移
                if update_offset_callback:
                    update_offset_callback(computed_offset)


            if success and ppt_filename:
                print(f"✓ 图片 {png_file.name} 处理完成，PPT窗口已创建: {ppt_filename}")
                
                # 从下载文件夹查找并复制PPT文件
                if " - PowerPoint" in ppt_filename:
                    base_filename = ppt_filename.replace(" - PowerPoint", "").strip()
                else:
                    base_filename = ppt_filename.strip()
                
                if not base_filename.endswith(".pptx"):
                    search_filename = base_filename + ".pptx"
                else:
                    search_filename = base_filename
                
                ppt_source_path = downloads_folder / search_filename
                
                if not ppt_source_path.exists():
                    print(f"  未找到 {ppt_source_path}，尝试查找最近的.pptx文件...")
                    pptx_files = list(downloads_folder.glob("*.pptx"))
                    if pptx_files:
                        pptx_files.sort(key=lambda x: x.stat().st_mtime, reverse=True)
                        ppt_source_path = pptx_files[0]
                        print(f"  找到最近的PPT文件: {ppt_source_path.name}")
                
                if ppt_source_path.exists():
                    target_filename = png_file.stem + ".pptx"
                    target_path = ppt_dir / target_filename
                    
                    shutil.copy2(ppt_source_path, target_path)
                    print(f"  ✓ PPT文件已复制: {target_path}")
                    
                    try:
                        ppt_source_path.unlink()
                        print(f"  ✓ 已删除原文件: {ppt_source_path}")
                    except Exception as e:
                        print(f"  ⚠ 删除原文件失败: {e}")
                else:
                    print(f"  ⚠ 未在下载文件夹中找到PPT文件")
            elif success:
                print(f"✓ 图片 {png_file.name} 处理完成，但未获取到PPT文件名")
            else:
                print(f"⚠ 图片 {png_file.name} 已截图，但未检测到新的PPT窗口")
                close_button = (display_width - 35, display_height + 35)
                mouse.click(button='left', coords=close_button)
        except Exception as e:
            print(f"✗ 处理图片 {png_file.name} 时出错: {e}")
        finally:
            stop_event.set()
            viewer_thread.join(timeout=2)
        
        if idx < len(png_files):
            print(f"等待 {delay_between_images} 秒后处理下一张...")
            time.sleep(delay_between_images)
    
    print("\n" + "=" * 60)
    print(f"完成! 共处理 {len(png_files)} 张图片")
    print("=" * 60)
    return png_names


def main():
    # 如果没有参数，或者第一个参数是 --gui，则启动 GUI
    if len(sys.argv) == 1 or (len(sys.argv) > 1 and sys.argv[1] == "--gui"):
        from .gui import launch_gui
        launch_gui()
        return

    # 删除CLI
    print("命令行模式已被弃用，请使用 GUI 界面。")
    

if __name__ == "__main__":
    main()



import json
import numpy as np
import cv2
from PIL import Image
import os
import requests
from spire.presentation.common import *
from spire.presentation import *
from .ppt_combiner import clean_ppt

def recursive_blocks(blocks):
    result = []
    for block in blocks:
        if "blocks" in block:
            result.extend(recursive_blocks(block["blocks"]))
        else:
            result.append(block)
    return result


def get_scaled_para_blocks(resize_scale, pdf_info, page_index, cond = 'no_image'):
    para_blocks = pdf_info[page_index]['para_blocks'] + pdf_info[page_index]['discarded_blocks']
    para_blocks = recursive_blocks(para_blocks)

    scaled_para_blocks = []
    for block in para_blocks:
        if cond == 'no_image' and block['type'] in ['image_body',"table_body"]:
            continue
        if cond == 'only_image' and block['type'] not in ['image_body',"table_body"]:
            continue
        scaled_block = block.copy()
        bbox = block['bbox']
        scaled_bbox = [
            bbox[0] * resize_scale,
            bbox[1] * resize_scale,
            bbox[2] * resize_scale,
            bbox[3] * resize_scale
        ]
        scaled_block['bbox'] = scaled_bbox
        scaled_para_blocks.append(scaled_block)
    return scaled_para_blocks



def compute_iou(boxA, boxB):
    # box = [left, top, right, bottom]
    xA = max(boxA[0], boxB[0])
    yA = max(boxA[1], boxB[1])
    xB = min(boxA[2], boxB[2])
    yB = min(boxA[3], boxB[3])

    interWidth = max(0, xB - xA)
    interHeight = max(0, yB - yA)
    interArea = interWidth * interHeight

    boxAArea = (boxA[2] - boxA[0]) * (boxA[3] - boxA[1])
    boxBArea = (boxB[2] - boxB[0]) * (boxB[3] - boxB[1])

    iou = interArea / float(boxAArea + boxBArea - interArea)

    return iou

def compute_ious(left, top, height, width, scaled_para_blocks):
    bbox = [left, top, left + width, top + height]
    ious = []
    for block in scaled_para_blocks:
        block_bbox = block['bbox']
        iou = compute_iou(bbox, block_bbox)
        ious.append(iou)
    return ious

def download_image(image_url, tmp_image_path):
    if os.path.exists(tmp_image_path):
        return
    response = requests.get(image_url)

    with open(tmp_image_path, 'wb') as f:
        f.write(response.content)

def load_json(file_path):
    with open(file_path, 'r', encoding='utf-8') as f:
        data = json.load(f)
    return data



def compute_edge_diversity(image_cv, left, top, right, bottom):
    left, top, right, bottom = int(left), int(top), int(right), int(bottom)
    top_edge = image_cv[top:top+1, left:right]
    bottom_edge = image_cv[bottom-1:bottom, left:right]
    left_edge = image_cv[top:bottom, left:left+1]
    right_edge = image_cv[top:bottom, right-1:right]
    edges = [top_edge, bottom_edge, left_edge, right_edge]

    diversity = np.max([edge.astype(np.float32).reshape(-1, 3).std(axis=0).mean() for edge in edges])# (N, 3)
    
    return diversity


def compute_color_diff(color1, color2):
    # CIELAB - 避免 uint8 在相减时发生环绕（underflow/overflow），先转换为 float
    # color1 is [R,G,B]
    # color2 is [R,G,B]
    color1_lab = cv2.cvtColor(np.uint8([[color1]]), cv2.COLOR_RGB2Lab)[0][0].astype(np.float32)
    color2_lab = cv2.cvtColor(np.uint8([[color2]]), cv2.COLOR_RGB2Lab)[0][0].astype(np.float32)
    diff = np.linalg.norm(color1_lab - color2_lab)
    return float(diff)


def compute_four_point_diff(image_cv, left, top, right, bottom):
    left, top, right, bottom = int(left), int(top), int(right), int(bottom)
    top_left = image_cv[top, left]
    top_right = image_cv[top, right-1]
    bottom_left = image_cv[bottom-1, left]
    bottom_right = image_cv[bottom-1, right-1]

    diffs = [
        compute_color_diff(top_left, top_right),
        compute_color_diff(top_left, bottom_left),
        compute_color_diff(bottom_right, top_right),
        compute_color_diff(bottom_right, bottom_left),
    ]
    return np.mean(diffs)


def get_indices_from_png_names(png_names):
    indices = []
    for name in png_names:
        base = os.path.basename(name)
        index_str = base.replace('page_', '').replace('.png', '')
        indices.append(int(index_str) - 1)
    return indices


def refine_ppt(tmp_image_dir, json_file, ppt_file, png_dir, png_files, final_out_ppt_file):
    png_files = [os.path.join(png_dir, name) for name in png_files]
    indices = get_indices_from_png_names(png_files)
    os.makedirs(tmp_image_dir, exist_ok=True)
    data = load_json(json_file)
    pdf_info = data['pdf_info']

    pdf_info = [pdf_info[i] for i in indices] # 只保留需要的页码信息

    pdf_w, _ = pdf_info[0]['page_size']
    

    presentation = Presentation()
    presentation.LoadFromFile(ppt_file)

    ppt_H, ppt_W = presentation.SlideSize.Size.Height, presentation.SlideSize.Size.Width

    ppt_scale = ppt_W / pdf_w

    assert len(png_files) == len(pdf_info) == len(presentation.Slides)
    
    for page_index, slide in enumerate(presentation.Slides):
        print(f"优化 第 {page_index+1}/{len(png_files)} 页...")
        scaled_para_blocks = get_scaled_para_blocks(ppt_scale,pdf_info, page_index)
        # 删除不相关文本框, 统一字体
        for i in range(slide.Shapes.Count - 1, -1, -1):
            # Check if those shapes are images
            shape = slide.Shapes[i]
            print("---")
            if "IAutoShape" not in str(type(shape)):
                slide.Shapes.RemoveAt(i) # 删除非文本框形状
                continue
            # Get the first paragraph of the shape
            paragraph = shape.TextFrame.Paragraphs[0]        

            left, top, text, width, height = shape.Left,shape.Top, shape.TextFrame.Text,shape.Width,shape.Height
            print(f"text:{text} left:{left} top:{top} width:{width} height:{height}")
            ious = compute_ious(left, top, height, width, scaled_para_blocks)
            # print(len(ious))

            if np.max(ious)>0.01:
                print("max iou:",np.max(ious))
                
                neareast_block = scaled_para_blocks[np.argmax(ious)]
                if neareast_block['type'] in ['title','text']:
                    print(neareast_block)
            else:
                print("invalid")
                slide.Shapes.RemoveAt(i)
                continue


            assert left+width <= ppt_W +10
            assert top+height <= ppt_H +10

            # Create a font
            newFont = TextFont("微软雅黑")

            # Loop through the text ranges in the paragraph
            for textRange in paragraph.TextRanges:
                textRange.LatinFont = newFont # 更换字体

        # 替换图片    
        image_blocks = get_scaled_para_blocks(ppt_scale,pdf_info, page_index,'only_image')
        for image_block in image_blocks:
            for line in image_block['lines']:
                for span in line['spans']:
                    tmp_image_path = os.path.join(tmp_image_dir, os.path.basename(span['image_path']))

                    download_image(span['image_path'], tmp_image_path)

                    left, top, right, bottom = image_block['bbox']

                    rect1 = RectangleF.FromLTRB(left, top, right, bottom)
                    image = slide.Shapes.AppendEmbedImageByPath(ShapeType.Rectangle, tmp_image_path, rect1)
                    image.Line.FillType = FillFormatType.none
                    image.ZOrderPosition = 0  # 设置图片在最底层
        

        # 替换背景    
        background = slide.SlideBackground
        old_bg_file = "old_bg.png"
        try:
            background.Fill.PictureFill.Picture.EmbedImage.Image.Save(old_bg_file)
            old_bg_cv = np.array(Image.open(old_bg_file))
            os.remove(old_bg_file)
        except:
            print("No existing background image found in slide ", page_index)
            old_bg_cv = None
        # 替换背景    
        background.Type = BackgroundType.Custom

        # Set the fill mode of the slide's background as a picture fill
        background.Fill.FillType = FillFormatType.Picture

        # Add an image to the image collection of the presentation

        png_file = png_files[page_index]
        image_cv = Image.open(png_file)
        image_cv = np.array(image_cv)

        image_h, image_w, _ = image_cv.shape

        if old_bg_cv is not None:
            old_bg_cv = cv2.resize(old_bg_cv, (image_w, image_h), interpolation=cv2.INTER_CUBIC)

        image_scale = image_w / pdf_w

        text_blocks = get_scaled_para_blocks(image_scale, pdf_info, page_index, cond='no_image')

        # mask = np.zeros(image_cv.shape[:-1], dtype=bool)

        for text_block in text_blocks:
            bbox = text_block['bbox']
            
            l, t, r, b = map(int, bbox)
            # 取左上角和右下角颜色平均值
            fill_color = image_cv[t, l] * 0.5 + image_cv[b, r] * 0.5
            fill_color = fill_color.astype(np.uint8).tolist()
            diversity = compute_edge_diversity(image_cv, l, t, r, b)
            # print(diversity)
            
            diff = compute_four_point_diff(image_cv, l, t, r, b)
            print("div=", diversity, " diff=", diff, " text_block=", text_block)
            if old_bg_cv is None or (diversity < 10 and diff < 9): # 边缘多样性低，认为是纯色区域，则可以直接填充
                cv2.rectangle(image_cv, (l, t), (r, b), fill_color, thickness=-1)
            else: # 边缘多样性高，保留原背景
                image_cv[t:b, l:r] = old_bg_cv[t:b, l:r] # 保留原背景的前提是要有原背景图

        tmp_bg_file = png_file.replace('.png', '_bg.png')
        Image.fromarray(image_cv).save(tmp_bg_file)
        stream = Stream(tmp_bg_file)

        imageData = presentation.Images.AppendStream(stream)
        # Set the image as the slide's background
        background.Fill.PictureFill.FillType = PictureFillType.Stretch
        background.Fill.PictureFill.Picture.EmbedImage = imageData
        
    presentation.SaveToFile(final_out_ppt_file, FileFormat.Pptx2019)

    print(f"优化完成! 输出文件: {final_out_ppt_file}")
    clean_ppt(final_out_ppt_file,final_out_ppt_file)





# -*- coding: utf-8 -*-
# D:/research/ICCV/vis_pixel_fake_rebuttal3/具象抽象_模型建构_Page1.png

# 使用 OpenCV 打开图片，按屏幕比例缩放并全屏显示；无边框/工具栏。

import os
import sys
import ctypes
import cv2
import numpy as np
from PIL import Image


def _get_screen_resolution():
    # Windows 获取屏幕分辨率，并启用 DPI 感知避免缩放影响
    try:
        user32 = ctypes.windll.user32
        try:
            user32.SetProcessDPIAware()
        except Exception:
            pass
        return user32.GetSystemMetrics(0), user32.GetSystemMetrics(1)
    except Exception:
        return 1920, 1080


def show_image_fullscreen(image_path: str, display_height: int = None):
    """
    显示图片在屏幕左上角
    
    Args:
        image_path: 图片路径
        display_height: 指定显示高度（像素），如果为None则自动适配屏幕
    """
    if not os.path.exists(image_path):
        raise FileNotFoundError(f"未找到图片: {image_path}")
    img = Image.open(image_path).convert("RGB")
    # img = cv2.imread(image_path, cv2.IMREAD_UNCHANGED)
    img = np.array(img)[...,::-1]  # RGB to BGR
    if img is None:
        raise ValueError(f"无法读取图片: {image_path}")

    screen_w, screen_h = _get_screen_resolution()
    h, w = img.shape[:2]

    # 根据指定高度或屏幕大小计算缩放比例
    if display_height is not None:
        scale = display_height / h
    else:
        scale = min(screen_w / w, screen_h / h)
    
    new_w = max(1, int(w * scale))
    new_h = max(1, int(h * scale))

    interp = cv2.INTER_AREA if scale < 1.0 else cv2.INTER_CUBIC
    resized = cv2.resize(img, (new_w, new_h), interpolation=interp)

    # 放置到左上角（不居中）
    pad_left = 0
    pad_right = max(0, screen_w - new_w)
    pad_top = 0
    pad_bottom = max(0, screen_h - new_h)
    canvas = cv2.copyMakeBorder(
        resized, pad_top, pad_bottom, pad_left, pad_right,
        borderType=cv2.BORDER_CONSTANT, value=(0, 0, 0)
    )

    win_name = "__opencv_fullscreen__"
    cv2.namedWindow(win_name, cv2.WINDOW_NORMAL)
    cv2.setWindowProperty(win_name, cv2.WND_PROP_FULLSCREEN, cv2.WINDOW_FULLSCREEN)
    cv2.imshow(win_name, canvas)

    # 置顶窗口（Windows）
    try:
        hwnd = ctypes.windll.user32.FindWindowW(None, win_name)
        if hwnd:
            # SetWindowPos(HWND_TOPMOST= -1)
            ctypes.windll.user32.SetWindowPos(
                hwnd,
                -1,
                0,
                0,
                0,
                0,
                0x0002 | 0x0001  # SWP_NOSIZE | SWP_NOMOVE
            )
            # 强制激活窗口，避免第二次被压到后台
            ctypes.windll.user32.ShowWindow(hwnd, 9)  # SW_RESTORE
            ctypes.windll.user32.SetForegroundWindow(hwnd)
            ctypes.windll.user32.BringWindowToTop(hwnd)
    except Exception:
        pass

    # # 按任意键或 ESC 退出
    # key = cv2.waitKey(0)
    # if key == 27:  # ESC
    #     pass
    # cv2.destroyAllWindows()


if __name__ == "__main__":
    path = sys.argv[1] if len(sys.argv) > 1 else "具象抽象_模型建构_Page1.png"
    # 如果提供了第二个参数，则作为显示高度
    height = int(sys.argv[2]) if len(sys.argv) > 2 else None
    show_image_fullscreen(path, display_height=height)


"""自动化微软电脑管家的智能圈选功能。

步骤:
1) 发送 Ctrl+Shift+A 打开智能圈选工具
2) 从左上角拖动到右下角选择全屏
3) 点击完成按钮以保存截图为 PPT

运行此脚本时，需要确保要捕获的屏幕可见。

注意：默认使用微软电脑管家的智能圈选功能（快捷键：Ctrl+Shift+A）
"""

import re
import time
import threading
import cv2
import win32api
import win32gui
import win32con
from pywinauto import mouse, keyboard
from pathlib import Path
from tkinter import messagebox
import tkinter as tk
# Get screen dimensions
screen_width = win32api.GetSystemMetrics(0)
screen_height = win32api.GetSystemMetrics(1)


def _wait_for_left_click(timeout: float = 60.0):
    """等待用户进行一次左键点击，返回点击时的屏幕坐标 (x, y)。超时返回 None。"""
    print(f"请手动点击“完成”按钮以保存第一页（等待 {int(timeout)} 秒）...")
    start = time.time()
    prev_state = bool(win32api.GetAsyncKeyState(0x01) & 0x8000)
    while time.time() - start < timeout:
        state = bool(win32api.GetAsyncKeyState(0x01) & 0x8000)
        # 检测按下的瞬间
        if state and not prev_state:
            # 记录按下时的坐标
            x, y = win32api.GetCursorPos()
            # 等待释放
            while bool(win32api.GetAsyncKeyState(0x01) & 0x8000):
                time.sleep(0.01)
            print(f"检测到左键点击坐标: {x}, {y}")
            return x, y
        prev_state = state
        time.sleep(0.01)
    print("等待用户点击超时。")
    return None


def get_ppt_windows():
    """获取当前所有PowerPoint窗口的句柄列表"""
    ppt_windows = []
    
    def enum_callback(hwnd, results):
        if win32gui.IsWindowVisible(hwnd):
            window_text = win32gui.GetWindowText(hwnd)
            class_name = win32gui.GetClassName(hwnd)
            # PowerPoint 窗口类名通常为 "PPTFrameClass"
            if "PPTFrameClass" in class_name or "PowerPoint" in window_text:
                results.append(hwnd)
        return True
    
    win32gui.EnumWindows(enum_callback, ppt_windows)
    return ppt_windows


def get_explorer_windows():
    """获取当前所有文件资源管理器窗口的句柄列表"""
    explorer_windows = []
    
    def enum_callback(hwnd, results):
        if win32gui.IsWindowVisible(hwnd):
            window_text = win32gui.GetWindowText(hwnd)
            class_name = win32gui.GetClassName(hwnd)
            # 文件资源管理器窗口类名通常为 "CabinetWClass"
            if "CabinetWClass" in class_name:
                results.append((hwnd, window_text))
        return True
    
    win32gui.EnumWindows(enum_callback, explorer_windows)
    return explorer_windows


def check_new_ppt_window(initial_windows, timeout=30, check_interval=1):
    """
    检查是否出现新的PPT窗口
    
    参数:
        initial_windows: 初始的PPT窗口句柄列表
        timeout: 超时时间（秒），默认30秒
        check_interval: 检查间隔（秒），默认1秒
    
    返回:
        (bool, list, str): (是否找到新窗口, 新窗口句柄列表, PPT文件名)
    """
    print(f"\n开始监测新的PowerPoint窗口 (超时时间: {timeout}秒)...")
    start_time = time.time()
    detected_new_window = False
    last_loading_window = None  # 最后一个"正在打开"的窗口
    seen_windows = set(initial_windows)  # 追踪所有见过的窗口
    
    while time.time() - start_time < timeout:
        current_windows = get_ppt_windows()
        new_windows = [w for w in current_windows if w not in seen_windows]
        
        # 更新已见过的窗口列表
        seen_windows.update(new_windows)
        
        if new_windows or detected_new_window:
            if new_windows and not detected_new_window:
                elapsed = time.time() - start_time
                print(f"✓ 检测到 {len(new_windows)} 个新的PowerPoint窗口 (耗时: {elapsed:.1f}秒)")
                detected_new_window = True
            
            # 检查所有当前窗口（不只是新窗口），因为窗口标题可能会更新
            all_new_windows = [w for w in current_windows if w not in initial_windows]
            
            for hwnd in all_new_windows:
                try:
                    window_text = win32gui.GetWindowText(hwnd)
                except:
                    continue
                
                # 检查是否是临时加载状态
                is_loading = window_text and ("正在打开" in window_text or "Opening" in window_text)
                
                if is_loading:
                    if hwnd != last_loading_window:
                        last_loading_window = hwnd
                        print(f"  - 检测到窗口正在加载: {window_text}，等待完全加载...")
                    continue
                
                # 找到有效的文件名（非空且不是加载状态）
                # 排除只有"PowerPoint"而没有文件名的情况
                if window_text and window_text.strip():
                    # 如果窗口标题只是"PowerPoint"，说明文件名还没有加载，继续等待
                    if window_text.strip().lower() == "powerpoint":
                        if hwnd != last_loading_window:
                            last_loading_window = hwnd
                            print(f"  - 窗口标题尚未完全加载（仅显示'PowerPoint'），继续等待...")
                        continue
                    
                    print(f"  ✓ 窗口加载完成: {window_text}")
                    
                    # 如果是SmartCopy窗口，识别后自动关闭
                    if "smartcopy" in window_text.lower():
                        try:
                            win32gui.PostMessage(hwnd, win32con.WM_CLOSE, 0, 0)
                            print(f"  → SmartCopy窗口已识别并关闭")
                        except Exception as e:
                            print(f"  → 关闭SmartCopy窗口失败: {e}")
                    
                    return True, all_new_windows, window_text
        
        remaining = timeout - (time.time() - start_time)
        if remaining > 0:
            if detected_new_window:
                print(f"  等待窗口标题更新... (剩余: {remaining:.0f}秒)", end='\r')
            else:
                print(f"  等待中... (剩余: {remaining:.0f}秒)", end='\r')
            time.sleep(check_interval)
    
    # 超时了，但如果检测到了"正在打开"的窗口，返回成功但文件名为None
    # 这样调用方可以尝试查找最近的文件
    if detected_new_window:
        print(f"\n⚠ 窗口标题未更新，将尝试查找最近的PPT文件")
        all_new_windows = [w for w in get_ppt_windows() if w not in initial_windows]
        return True, all_new_windows, None
    
    print(f"\n✗ 在 {timeout} 秒内未检测到新的PowerPoint窗口")
    return False, [], None


def check_and_close_download_folder(initial_explorer_windows, timeout=10, check_interval=0.5):
    """
    检查是否出现新的文件资源管理器窗口（特别是下载文件夹），如果有则关闭
    
    参数:
        initial_explorer_windows: 初始的文件资源管理器窗口列表 [(hwnd, title), ...]
        timeout: 超时时间（秒），默认10秒
        check_interval: 检查间隔（秒），默认0.5秒
    
    返回:
        int: 关闭的窗口数量
    """
    print(f"\n开始监测新的文件资源管理器窗口 (超时时间: {timeout}秒)...")
    start_time = time.time()
    closed_count = 0
    initial_hwnds = [hwnd for hwnd, _ in initial_explorer_windows]
    
    while time.time() - start_time < timeout:
        current_windows = get_explorer_windows()
        new_windows = [(hwnd, title) for hwnd, title in current_windows if hwnd not in initial_hwnds]
        
        if new_windows:
            for hwnd, title in new_windows:
                try:
                    # 检查是否是下载文件夹（标题通常包含"下载"或"Downloads"）
                    is_download_folder = "下载" in title or "Downloads" in title
                    
                    print(f"✓ 检测到新的文件资源管理器窗口: {title}")
                    if is_download_folder:
                        print(f"  → 检测到下载文件夹，正在关闭...")
                    
                    # 关闭新窗口（无论是否是下载文件夹，都关闭新打开的资源管理器）
                    win32gui.PostMessage(hwnd, win32con.WM_CLOSE, 0, 0)
                    closed_count += 1
                    print(f"  → 已发送关闭指令")
                    
                    # 将已处理的窗口加入初始列表，避免重复处理
                    initial_hwnds.append(hwnd)
                    
                except Exception as e:
                    print(f"  → 关闭窗口失败: {e}")
        
        remaining = timeout - (time.time() - start_time)
        if remaining > 0:
            time.sleep(check_interval)
    
    if closed_count > 0:
        print(f"\n✓ 共关闭 {closed_count} 个文件资源管理器窗口")
    else:
        print(f"\n✓ 未检测到新的文件资源管理器窗口")
    
    return closed_count


def create_topmost_dialog():    
    # 创建一个简单的Tkinter窗口    
    root = tk.Tk()    
    root.withdraw()  # 隐藏主窗口    
    # 设置窗口为置顶    
    root.attributes("-topmost", True)   
    msg = (
            "正在进行按钮位置校准，需要您的一次交互确认，请仔细阅读此说明\n\n"
            "点击“确定”后不要动鼠标，等到出现智能圈选工具栏后，请您移动鼠标手动点击“智能复制到PPT”按钮即可完整校准。\n\n"
            "请在准备好后点击“确定”继续。转换过程中请不要最小化窗口或干扰鼠标操作。"
    )
    a = messagebox.askokcancel('提示', msg, parent=root)
    root.destroy() 



def take_fullscreen_snip(
    delay_before_hotkey: float = 1.0,
    drag_duration: float = 3,
    click_duration: float = 0.1,
    check_ppt_window: bool = True,
    ppt_check_timeout: float = 30,
    width: int = screen_width,
    height: int = screen_height,
    done_button_right_offset: int | None = None,
):
    """使用微软电脑管家的智能圈选功能进行全屏截图。

    Args:
        delay_before_hotkey: 发送 Ctrl+Shift+A 之前等待的秒数
        drag_duration: 拖动操作持续的秒数（模拟等待）
        click_duration: 点击完成按钮的秒数
        check_ppt_window: 是否检查新的PPT窗口，默认True
        ppt_check_timeout: PPT窗口检测超时时间（秒），默认30
        width: 截图宽度，默认为屏幕宽度
        height: 截图高度，默认为屏幕高度
        done_button_right_offset: 完成按钮的右侧偏移量（像素），用于手动覆盖
        pc_manager_version: 电脑管家版本号；3.19及以上自动使用 190，低于3.19 使用 210
    
    Returns:
        tuple: (bool, str, int|None) - (是否成功检测到新窗口, PPT文件名, 已保存的偏移或 None)
               如果不需要检查PPT窗口，返回 (True, None, None)
    """
    
    # 记录点击前的PPT窗口和文件资源管理器窗口
    initial_ppt_windows = get_ppt_windows() if check_ppt_window else []
    initial_explorer_windows = get_explorer_windows()
    
    if check_ppt_window:
        print(f"点击前PPT窗口数量: {len(initial_ppt_windows)}")
    print(f"点击前文件资源管理器窗口数量: {len(initial_explorer_windows)}")

    # 等待用户聚焦到正确的窗口
    time.sleep(delay_before_hotkey)


    # 简化逻辑：优先使用函数参数；如果未提供或强制重新捕获，则要求手动点击以捕获偏移并保存
    resolved_offset = None
    computed_offset = None
    if done_button_right_offset is not None:
        resolved_offset = int(done_button_right_offset)
        print(f"使用传入的完成按钮偏移: {resolved_offset}")
    else:
        print("未传入完成按钮偏移，稍后将要求手动点击以捕获并保存偏移。")
        create_topmost_dialog()

    # 打开微软电脑管家的智能圈选工具
    # pywinauto 使用 '^+a' 表示 Ctrl+Shift+A
    keyboard.send_keys('^+a')
    time.sleep(2)

    # Define key points for the snip and confirmation click.
    # top_left = (5, 5)
    top_left = (0,0)
    # delta = 4  # Small offset to ensure full coverage
    delta = int(width / 512 * 4)
    bottom_right = (width+delta, height)

    center = (width // 2, height // 2)

    print(bottom_right, width)

    # Perform the drag operation
    # Move to start position
    mouse.move(coords=top_left)
    
    # Press left button
    mouse.press(button='left', coords=top_left)
    
    # Wait for the duration to simulate the drag time

    time.sleep(1)
    


    # Release left button
    mouse.release(button='left', coords=bottom_right)
    # 如果没有传入偏移或强制重新捕获，则要求手动点击完成按钮以捕获偏移并保存
    if resolved_offset is None:
        

        
        
        coords = _wait_for_left_click(timeout=60)
        if coords:
            click_x, click_y = coords
            computed_offset = int((bottom_right[0]) - click_x)
            print(f"已计算并保存完成按钮偏移: {computed_offset}")
            resolved_offset = computed_offset
        else:
            print("首次偏移捕获超时或未检测到点击，放弃操作。")
            return False, None, None
    else:
        # 已有偏移，执行自动点击
        done_button = (bottom_right[0] - resolved_offset, height + 35)
        if done_button[1] > screen_height:
            done_button = (done_button[0], height - 35)
        mouse.move(coords=done_button)
        time.sleep(1)
        mouse.click(button='left', coords=done_button)
    
    # 检查是否出现新的PPT窗口
    if check_ppt_window:
        success, new_windows, ppt_filename = check_new_ppt_window(initial_ppt_windows, timeout=ppt_check_timeout)
        
        # 同时检查并关闭新打开的文件资源管理器窗口（下载文件夹）
        check_and_close_download_folder(initial_explorer_windows, timeout=10)
        
        return success, ppt_filename, computed_offset
    
    return True, None, None

if __name__ == "__main__":
    from .image_viewer import show_image_fullscreen

    image_path = "Hackathon_Architect_Playbook_pngs/page_0001.png"

    stop_event = threading.Event()

    def _viewer():
        # 打开全屏窗口
        show_image_fullscreen(image_path)
        # 维持 OpenCV 事件循环，否则窗口可能不刷新
        while not stop_event.is_set():
            # 处理 GUI 事件；保持窗口响应
            cv2.waitKey(50)
        # 停止时关闭窗口
        try:
            cv2.destroyAllWindows()
        except Exception:
            pass

    t = threading.Thread(target=_viewer, name="opencv_viewer", daemon=True)
    t.start()

    # 等待窗口稳定后开始截图
    time.sleep(2)
    try:
        take_fullscreen_snip()
    finally:
        # 通知关闭窗口并等待线程退出
        stop_event.set()
        t.join(timeout=2)



import cv2
# from pyinpaint import Inpaint
import numpy as np
from skimage import data
from skimage.morphology import disk, binary_dilation
from skimage.restoration import inpaint
from PIL import Image
org_img = "Hackathon_Architect_Playbook_pngs/page_0001.png"
# mask = "page_0001_mask.jpg"



def inpaint_image(image_path, output_path):
    # 2867, 1600
    image = Image.open(image_path)
    image_defect = np.array(image)
    mask = np.zeros(image_defect.shape[:-1], dtype=bool)
    # [{\"width\":240,\"top\":1530,\"height\":65,\"left\":2620}]
    r1,r2,c1,c2 = 1530,1595,2620,2860

    old_width, old_height = 2867,1600

    image_width, image_height = image_defect.shape[1], image_defect.shape[0]
    ratio = image_width / old_width

    assert abs(ratio - (image_height / old_height)) < 0.01, "图片比例不对，无法修复"


    r1 = int(r1 * ratio)
    r2 = int(r2 * ratio)
    c1 = int(c1 * ratio)
    c2 = int(c2 * ratio)

    mask[r1:r2, c1:c2] = True

    image_result = inpaint.inpaint_biharmonic(image_defect, mask, channel_axis=-1)
    # print(image_result)
    # cv2.imwrite(output_path, 
    Image.fromarray((image_result*255).astype("uint8")).save(output_path)


import tkinter as tk
from tkinter import ttk, filedialog, messagebox, scrolledtext
import threading
import sys
import os
import windnd
from pathlib import Path
from .cli import process_pdf_to_ppt
from .utils.ppt_combiner import combine_ppt
from .utils.screenshot_automation import screen_width, screen_height
from .utils.ppt_refiner import refine_ppt
import json
import ctypes
import webbrowser

MINERU_URL = "https://mineru.net/"
GITHUB_URL = "https://github.com/elliottzheng/NotebookLM2PPT"


CONFIG_FILE = Path("./config.json")


BASE_WINDOWS_DPI = 85


def enable_windows_dpi_awareness(root=None):
    """Enable DPI awareness on Windows and adjust Tk scaling.

    Call this before creating UI or immediately after creating `root`.
    This helps fix display issues when Windows scaling is 200%.
    """
    if sys.platform != "win32":
        return
    # Try modern APIs first, fall back gracefully
    try:
        user32 = ctypes.windll.user32
        # Try SetProcessDpiAwarenessContext (Windows 10+)
        if hasattr(user32, 'SetProcessDpiAwarenessContext'):
            try:
                # DPI_AWARENESS_CONTEXT_PER_MONITOR_AWARE_V2 = -4
                user32.SetProcessDpiAwarenessContext(ctypes.c_void_p(-4))
            except Exception:
                pass
        else:
            # Try shcore.SetProcessDpiAwareness(PROCESS_PER_MONITOR_DPI_AWARE=2)
            try:
                shcore = ctypes.windll.shcore
                shcore.SetProcessDpiAwareness(2)
            except Exception:
                # Older fallback
                try:
                    user32.SetProcessDPIAware()
                except Exception:
                    pass

        # If a root was provided, try to set tk scaling to system DPI
        if root is not None:
            try:
                # Get system DPI (fallback to 96)
                dpi = BASE_WINDOWS_DPI
                
                if hasattr(user32, 'GetDpiForSystem'):
                    dpi = user32.GetDpiForSystem()
                elif hasattr(user32, 'GetDeviceCaps'):
                    # Last resort: get DC dpi
                    hdc = user32.GetDC(0)
                    # LOGPIXELSX = 88
                    gdi32 = ctypes.windll.gdi32
                    dpi = gdi32.GetDeviceCaps(hdc, 88)
                scaling = float(dpi) / BASE_WINDOWS_DPI
                print("??", scaling)
                root.tk.call('tk', 'scaling', scaling)
            except Exception:
                pass
    except Exception:
        pass


class TextRedirector:
    def __init__(self, widget, tag="stdout"):
        self.widget = widget
        self.tag = tag

    def write(self, str):
        self.widget.configure(state='normal')
        self.widget.insert(tk.END, str, (self.tag,))
        self.widget.see(tk.END)
        self.widget.configure(state='disabled')

    def flush(self):
        pass

class AppGUI:
    def __init__(self, root):
        self.root = root
        self.root.title("NotebookLM2PPT - PDF 转 PPT 工具")
        self.root.geometry("850x750")
        self.root.minsize(750, 550)
        
        self.show_startup_dialog()
        
        self.setup_ui()
        
        # Save original stdout/stderr
        self.old_stdout = sys.stdout
        self.old_stderr = sys.stderr
        
        # Redirect stdout and stderr
        sys.stdout = TextRedirector(self.log_area, "stdout")
        sys.stderr = TextRedirector(self.log_area, "stderr")
        
        if windnd:
            windnd.hook_dropfiles(self.root, func=self.on_drop_files)
        
        self.root.protocol("WM_DELETE_WINDOW", self.on_closing)

    def on_drop_files(self, files):
        if files:
            file_path = files[0].decode('gbk') if isinstance(files[0], bytes) else files[0]
            lower_file_path = file_path.lower()
            if lower_file_path.endswith('.pdf'):
                self.pdf_path_var.set(file_path)
                print(f"已添加文件: {file_path}")
            elif lower_file_path.endswith('.json'):
                self.mineru_json_var.set(file_path)
            else:
                messagebox.showwarning("提示", "请拖拽 PDF 文件或者 Mineru JSON 文件！")

    def on_closing(self):
        self.dump_config_to_disk()
        sys.stdout = self.old_stdout
        sys.stderr = self.old_stderr
        self.root.destroy()

    def add_context_menu(self, widget):
        """为输入框添加右键菜单（剪切、复制、粘贴、全选）"""
        menu = tk.Menu(widget, tearoff=0)
        menu.add_command(label="剪切", command=lambda: widget.event_generate("<<Cut>>"))
        menu.add_command(label="复制", command=lambda: widget.event_generate("<<Copy>>"))
        menu.add_command(label="粘贴", command=lambda: widget.event_generate("<<Paste>>"))
        menu.add_separator()
        menu.add_command(label="全选", command=lambda: widget.select_range(0, tk.END))
        
        def show_menu(event):
            menu.post(event.x_root, event.y_root)
        
        widget.bind("<Button-3>", show_menu)

    def setup_ui(self):
        main_frame = ttk.Frame(self.root, padding="10")
        main_frame.pack(fill=tk.BOTH, expand=True)
        main_frame.columnconfigure(0, weight=1)

        # File Selection
        file_frame = ttk.LabelFrame(main_frame, text="📁 文件设置（支持拖拽 PDF/ 对应的MinerU JSON 文件到窗口）", padding="10")
        file_frame.pack(fill=tk.X, pady=5)
        file_frame.columnconfigure(1, weight=1)

        ttk.Label(file_frame, text="PDF 文件:").grid(row=0, column=0, sticky=tk.W)
        self.pdf_path_var = tk.StringVar()
        pdf_entry = ttk.Entry(file_frame, textvariable=self.pdf_path_var, width=60)
        pdf_entry.grid(row=0, column=1, padx=5, sticky="ew")
        self.add_context_menu(pdf_entry)
        ttk.Button(file_frame, text="浏览...", command=self.browse_pdf).grid(row=0, column=2)

        ttk.Label(file_frame, text="输入PDF对应的MinerU JSON (可选，进一步优化效果):").grid(row=2, column=0, sticky=tk.W, pady=5)
        self.mineru_json_var = tk.StringVar(value="")
        mineru_entry = ttk.Entry(file_frame, textvariable=self.mineru_json_var, width=60)
        mineru_entry.grid(row=2, column=1, padx=5, pady=5, sticky="ew")
        self.add_context_menu(mineru_entry)
        ttk.Button(file_frame, text="浏览...", command=self.browse_json).grid(row=2, column=2, pady=5)
        ttk.Button(file_frame, text="说明", command=self.show_mineru_info).grid(row=2, column=3, pady=5, padx=5)

        ttk.Label(file_frame, text="输出目录:").grid(row=1, column=0, sticky=tk.W, pady=5)
        self.output_dir_var = tk.StringVar(value="workspace")
        output_entry = ttk.Entry(file_frame, textvariable=self.output_dir_var, width=60)
        output_entry.grid(row=1, column=1, padx=5, pady=5, sticky="ew")
        self.add_context_menu(output_entry)
        ttk.Button(file_frame, text="浏览...", command=self.browse_output).grid(row=1, column=2, pady=5)        

        # Options
        opt_frame = ttk.LabelFrame(main_frame, text="⚙️ 转换选项", padding="10")
        opt_frame.pack(fill=tk.X, pady=5)
        opt_frame.columnconfigure(1, weight=1)
        opt_frame.columnconfigure(3, weight=1)

        ttk.Label(opt_frame, text="图片清晰度 (DPI):").grid(row=0, column=0, sticky=tk.W)
        self.dpi_var = tk.IntVar(value=150)
        dpi_entry = ttk.Entry(opt_frame, textvariable=self.dpi_var, width=10)
        dpi_entry.grid(row=0, column=1, sticky=tk.W, padx=5)
        self.add_context_menu(dpi_entry)
        ttk.Label(opt_frame, text="（建议 150-300，越高越清晰）", foreground="gray").grid(row=0, column=2, sticky=tk.W, padx=5)

        ttk.Label(opt_frame, text="等待时间 (秒):").grid(row=1, column=0, sticky=tk.W, pady=5)
        self.delay_var = tk.IntVar(value=2)
        delay_entry = ttk.Entry(opt_frame, textvariable=self.delay_var, width=10)
        delay_entry.grid(row=1, column=1, sticky=tk.W, padx=5, pady=5)
        self.add_context_menu(delay_entry)
        ttk.Label(opt_frame, text="（每页加载后的等待时间）", foreground="gray").grid(row=1, column=2, sticky=tk.W, padx=5, pady=5)

        ttk.Label(opt_frame, text="超时时间 (秒):").grid(row=2, column=0, sticky=tk.W, pady=5)
        self.timeout_var = tk.IntVar(value=50)
        timeout_entry = ttk.Entry(opt_frame, textvariable=self.timeout_var, width=10)
        timeout_entry.grid(row=2, column=1, sticky=tk.W, padx=5, pady=5)
        self.add_context_menu(timeout_entry)
        ttk.Label(opt_frame, text="（单页最大处理时间）", foreground="gray").grid(row=2, column=2, sticky=tk.W, padx=5, pady=5)

        ttk.Label(opt_frame, text="窗口显示比例:").grid(row=3, column=0, sticky=tk.W, pady=5)
        self.ratio_var = tk.DoubleVar(value=0.8)
        ratio_entry = ttk.Entry(opt_frame, textvariable=self.ratio_var, width=10)
        ratio_entry.grid(row=3, column=1, sticky=tk.W, padx=5, pady=5)
        self.add_context_menu(ratio_entry)
        ttk.Label(opt_frame, text="（建议 0.7-0.9）", foreground="gray").grid(row=3, column=2, sticky=tk.W, padx=5, pady=5)

        self.inpaint_var = tk.BooleanVar(value=True)
        ttk.Checkbutton(opt_frame, text="去除水印（图像修复）", variable=self.inpaint_var).grid(row=4, column=0, columnspan=3, sticky=tk.W, pady=5)

        ttk.Separator(opt_frame, orient='horizontal').grid(row=5, column=0, columnspan=4, sticky="ew", pady=10)

        ttk.Label(opt_frame, text="页码范围:").grid(row=6, column=0, sticky=tk.W, pady=5)
        self.page_range_var = tk.StringVar(value="")
        page_range_entry = ttk.Entry(opt_frame, textvariable=self.page_range_var, width=30)
        page_range_entry.grid(row=6, column=1, columnspan=2, sticky="ew", padx=5, pady=5)
        self.add_context_menu(page_range_entry)
        ttk.Label(opt_frame, text="留空=全部，示例: 1-3,5,7-9", foreground="gray").grid(row=6, column=3, sticky=tk.W, padx=5, pady=5)

        ttk.Separator(opt_frame, orient='horizontal').grid(row=7, column=0, columnspan=4, sticky="ew", pady=10)

        ttk.Label(opt_frame, text="按钮偏移 (像素):").grid(row=8, column=0, sticky=tk.W, pady=5)
        self.done_offset_var = tk.StringVar(value="")
        done_offset_entry = ttk.Entry(opt_frame, textvariable=self.done_offset_var, width=10)
        done_offset_entry.grid(row=8, column=1, sticky=tk.W, padx=5, pady=5)
        self.add_context_menu(done_offset_entry)
        self.saved_offset_var = tk.StringVar(value="")
        ttk.Label(opt_frame, textvariable=self.saved_offset_var, foreground="blue").grid(row=8, column=2, sticky=tk.W, padx=5)
        
        ttk.Label(opt_frame, text="⚠️ 核心参数：程序通过模拟鼠标点击'转换为PPT'按钮实现转换", foreground="red").grid(row=9, column=0, columnspan=4, sticky=tk.W)
        ttk.Label(opt_frame, text="   如果无法准确定位按钮位置，核心功能将无法实现！可通过勾选“校准按钮位置”进行校准", foreground="red").grid(row=10, column=0, columnspan=4, sticky=tk.W)
        
        self.calibrate_var = tk.BooleanVar(value=True)
        cb = ttk.Checkbutton(opt_frame, text="校准按钮位置", variable=self.calibrate_var)
        cb.grid(row=11, column=0, columnspan=3, sticky=tk.W, pady=5)
        # ttk 不支持 foreground，用样式或 Label 实现红色提示
        ttk.Label(opt_frame, text="提示: 程序会自动保存校准结果，下次无需重复校准", foreground="red").grid(row=12, column=0, columnspan=4, sticky=tk.W)


        # Control
        ctrl_frame = ttk.Frame(main_frame, padding="10")
        ctrl_frame.pack(fill=tk.X)

        self.start_btn = ttk.Button(ctrl_frame, text="🚀 开始转换", command=self.start_conversion)
        self.start_btn.pack(side=tk.LEFT, padx=5)

        # Log Area
        log_frame = ttk.LabelFrame(main_frame, text="📋 运行日志", padding="5")
        log_frame.pack(fill=tk.BOTH, expand=True, pady=5)

        self.log_area = scrolledtext.ScrolledText(log_frame, state='disabled', height=15)
        self.log_area.pack(fill=tk.BOTH, expand=True)
        self.log_area.tag_config("stderr", foreground="red")

        self.load_config_from_disk()

    def browse_pdf(self):
        # 清理路径中的引号和空格，方便用户直接粘贴带引号的路径
        current_path = self.pdf_path_var.get().strip().strip('"')
        initial_dir = os.path.dirname(current_path) if current_path and os.path.exists(os.path.dirname(current_path)) else None
        
        filename = filedialog.askopenfilename(
            parent=self.root,
            title="选择 PDF 文件",
            initialdir=initial_dir,
            filetypes=[("PDF files", "*.pdf"), ("All files", "*.*")]
        )
        if filename:
            self.pdf_path_var.set(filename)

    def browse_output(self):
        # 清理路径中的引号和空格
        current_dir = self.output_dir_var.get().strip().strip('"')
        initial_dir = current_dir if current_dir and os.path.exists(current_dir) else None
        
        directory = filedialog.askdirectory(
            parent=self.root,
            title="选择输出目录",
            initialdir=initial_dir
        )
        if directory:
            self.output_dir_var.set(directory)

    def browse_json(self):
        current_path = self.mineru_json_var.get().strip().strip('"')
        initial_dir = os.path.dirname(current_path) if current_path and os.path.exists(os.path.dirname(current_path)) else None
        filename = filedialog.askopenfilename(
            parent=self.root,
            title="选择 Mineru JSON 文件",
            initialdir=initial_dir,
            filetypes=[("JSON files", "*.json"), ("All files", "*.*")]
        )
        if filename:
            self.mineru_json_var.set(filename)

    def start_conversion(self):
        pdf_path = self.pdf_path_var.get().strip().strip('"')
        output_dir = self.output_dir_var.get().strip().strip('"')
        
        self.pdf_path_var.set(pdf_path)
        self.output_dir_var.set(output_dir)

        if not pdf_path or not os.path.exists(pdf_path):
            messagebox.showerror("错误", "请先选择一个 PDF 文件")
            return

        self.start_btn.config(state=tk.DISABLED)
        threading.Thread(target=self.run_conversion, daemon=True).start()

    def dump_config_to_disk(self):
        config_data = {
            "output_dir": self.output_dir_var.get(),
            "dpi": self.dpi_var.get(),
            "delay": self.delay_var.get(),
            "timeout": self.timeout_var.get(),
            "ratio": self.ratio_var.get(),
            "inpaint": self.inpaint_var.get(),
            "done_offset": self.done_offset_var.get(),
        }
        try:
            with open(CONFIG_FILE, 'w', encoding='utf-8') as f:
                json.dump(config_data, f, ensure_ascii=False, indent=4)
            print("✅ 配置已保存到磁盘")
        except Exception as e:
            print(f"⚠️ 配置保存失败: {str(e)}")

    def load_config_from_disk(self):
        try:
            with open(CONFIG_FILE, 'r', encoding='utf-8') as f:
                config_data = json.load(f)
                self.output_dir_var.set(config_data.get("output_dir", "workspace"))
                self.dpi_var.set(config_data.get("dpi", 150))
                self.delay_var.set(config_data.get("delay", 2))
                self.timeout_var.set(config_data.get("timeout", 50))
                self.ratio_var.set(config_data.get("ratio", 0.8))
                self.inpaint_var.set(config_data.get("inpaint", True))
                offset_value = config_data.get("done_offset", "")
                self.update_offset_related_gui(offset_value)
        except Exception as e:
            print(f"⚠️ 配置加载失败: {str(e)}")
            self.dump_config_to_disk()
            print("已创建默认配置文件")


    def update_offset_disk(self, offset_value):
        self.done_offset_var.set(str(offset_value))
        self.dump_config_to_disk()
        self.update_offset_related_gui(offset_value)

    def update_offset_related_gui(self, done_offset_value=None):
        saved = done_offset_value
        is_valid = saved is not None and saved != ""
        if is_valid:
            self.saved_offset_var.set(f"已保存: {saved}px")
            if not self.done_offset_var.get().strip():
                self.done_offset_var.set(str(saved))
        else:
            self.saved_offset_var.set("未保存: 将自动校准")
        self.calibrate_var.set(not is_valid)
        
    def show_startup_dialog(self):
        config_file = Path("./config.json")
        show_dialog = True
        
        try:
            if config_file.exists():
                with open(config_file, 'r', encoding='utf-8') as f:
                    config_data = json.load(f)
                    show_dialog = not config_data.get("hide_startup_dialog", False)
        except Exception:
            pass
        
        if not show_dialog:
            return
        
        top = tk.Toplevel(self.root)
        top.title("欢迎使用")
        top.geometry("500x300")
        top.resizable(False, False)
        
        top.update_idletasks()
        width = top.winfo_width()
        height = top.winfo_height()
        x = (top.winfo_screenwidth() // 2) - (width // 2)
        y = (top.winfo_screenheight() // 2) - (height // 2)
        top.geometry(f'{width}x{height}+{x}+{y}')
        
        info_frame = ttk.Frame(top, padding="20")
        info_frame.pack(fill=tk.BOTH, expand=True)
        
        info_text = (
            "本软件是免费开源的 PDF 转 PPT 工具\n\n"
            "开发者：Elliott Zheng\n\n"
            "如果您感觉本软件对您有所帮助，请在项目GitHub上给个star或是介绍给您的朋友，谢谢。\n\n"
            "本软件免费开源，如果您是以付费的方式获得本软件，那么你应该是被骗了。[○･｀Д´･○]\n\n"
            "感谢使用本工具！"
        )
        
        txt = scrolledtext.ScrolledText(info_frame, wrap=tk.WORD, height=8)
        txt.pack(fill=tk.BOTH, expand=True)
        txt.insert(tk.END, info_text)
        txt.configure(state='disabled')
        
        btn_frame = ttk.Frame(info_frame)
        btn_frame.pack(fill=tk.X, pady=(15, 0))
        
        def open_github():
            try:
                webbrowser.open_new_tab(GITHUB_URL)
            except Exception as e:
                messagebox.showerror("错误", f"无法打开网页: {e}")
        
        def on_ok():
            top.destroy()
        
        def on_dont_show():
            try:
                with open(config_file, 'r', encoding='utf-8') as f:
                    config_data = json.load(f)
            except Exception:
                config_data = {}
            
            config_data["hide_startup_dialog"] = True
            
            try:
                with open(config_file, 'w', encoding='utf-8') as f:
                    json.dump(config_data, f, ensure_ascii=False, indent=4)
            except Exception as e:
                print(f"⚠️ 保存配置失败: {e}")
            
            top.destroy()
        
        ttk.Button(btn_frame, text="打开 GitHub 主页", command=open_github).pack(side=tk.LEFT, padx=5)
        ttk.Button(btn_frame, text="不再显示", command=on_dont_show).pack(side=tk.RIGHT, padx=5)
        ttk.Button(btn_frame, text="确定", command=on_ok).pack(side=tk.RIGHT, padx=5)
        
        top.transient(self.root)
        top.grab_set()
        self.root.wait_window(top)
        
    def show_mineru_info(self):
        info = (
            "MinerU 是一个可在线使用的文档解析工具。\n\n"
            "使用步骤：\n"
            "1. 在 MinerU 网站 https://mineru.net/ 上传你的 PDF，等待解析完成。\n"
            "2. 解析完成后下载生成的 JSON 文件。\n"
            "3. 在本程序的“输入PDF对应的MinerU JSON (可选)”中选择该 JSON 文件。\n\n"
            "说明：该 JSON 包含页面结构、文本和排版等信息；本程序会利用它进一步优化输出 PPT 的图像、背景和文本，从而提升生成效果。\n\n"
            "注意：请确保 JSON 与要转换的 PDF 对应，否则优化效果可能不正确。"
        )
        top = tk.Toplevel(self.root)
        top.title("关于 MinerU")
        top.geometry("640x360")
        txt = scrolledtext.ScrolledText(top, wrap=tk.WORD, height=12)
        txt.pack(fill=tk.BOTH, expand=True, padx=8, pady=(8,6))
        txt.insert(tk.END, info)
        txt.configure(state='disabled')
        btn_frame = ttk.Frame(top)
        btn_frame.pack(side=tk.BOTTOM, fill=tk.X, padx=8, pady=6)

        def open_mineru_website():
            try:
                webbrowser.open_new_tab(MINERU_URL)
            except Exception as e:
                messagebox.showerror("错误", f"无法打开网页: {e}")

        ttk.Button(btn_frame, text="打开 MinerU 网站", command=open_mineru_website).pack(side=tk.LEFT, padx=6)
        ttk.Button(btn_frame, text="关闭", command=top.destroy).pack(side=tk.LEFT, padx=6)
        
    def run_conversion(self):
        try:
            pdf_file = self.pdf_path_var.get()
            pdf_name = Path(pdf_file).stem
            workspace_dir = Path(self.output_dir_var.get())
            png_dir = workspace_dir / f"{pdf_name}_pngs"
            ppt_dir = workspace_dir / f"{pdf_name}_ppt"
            out_ppt_file = workspace_dir / f"{pdf_name}.pptx"
            tmp_image_dir = workspace_dir / "tmp_images"
            
            workspace_dir.mkdir(exist_ok=True, parents=True)

            offset_raw = self.done_offset_var.get().strip()
            done_offset = None
            if offset_raw:
                try:
                    done_offset = int(offset_raw)
                except ValueError:
                    raise ValueError("完成按钮偏移需填写整数或留空")

            ratio = min(screen_width/16, screen_height/9)
            max_display_width = int(16 * ratio)
            max_display_height = int(9 * ratio)

            display_width = int(max_display_width * self.ratio_var.get())
            display_height = int(max_display_height * self.ratio_var.get())

            print(f"开始处理: {pdf_file}")

            # 解析页范围
            def parse_page_range(range_str):
                if not range_str:
                    return None
                pages = set()
                for part in [p.strip() for p in range_str.split(',') if p.strip()]:
                    if '-' in part:
                        start_end = part.split('-')
                        if start_end[0] == '':
                            continue
                        start = int(start_end[0])
                        if start_end[1] == '':
                            pages.update(range(start, start + 10000))
                        else:
                            end = int(start_end[1])
                            if end >= start:
                                pages.update(range(start, end + 1))
                    else:
                        pages.add(int(part))
                return sorted(pages)

            pages_list = None
            try:
                pages_list = parse_page_range(self.page_range_var.get().strip())
            except Exception as e:
                raise ValueError("页范围格式错误，请使用 1-3,5,7- 类似格式")
            
            png_names = process_pdf_to_ppt(
                pdf_path=pdf_file,
                png_dir=png_dir,
                ppt_dir=ppt_dir,
                delay_between_images=self.delay_var.get(),
                inpaint=self.inpaint_var.get(),
                dpi=self.dpi_var.get(),
                timeout=self.timeout_var.get(),
                display_height=display_height,
                display_width=display_width,
                done_button_offset=done_offset,
                capture_done_offset=self.calibrate_var.get(),
                pages=pages_list,
                update_offset_callback=self.update_offset_disk
            )

            png_names = combine_ppt(ppt_dir, out_ppt_file, png_names=png_names)
            # 如果用户提供了 mineru JSON，则进行 refine_ppt 处理
            mineru_json = self.mineru_json_var.get().strip().strip('"')
            if mineru_json:
                if not os.path.exists(mineru_json):
                    print(f"⚠️ 提供的 MinerU JSON 文件不存在，跳过 PPT 优化: {mineru_json}")
                else:
                    refined_out = workspace_dir / f"{pdf_name}_优化.pptx"
                    print(f"开始利用MinerU信息优化 PPT: {mineru_json}")
                    refine_ppt(str(tmp_image_dir), mineru_json, str(out_ppt_file), str(png_dir), png_names, str(refined_out))
                    
                    print("✅ refine_ppt 完成")
                    extra_message = "优化前的PPT已保存在同一目录下"
                    out_ppt_file = os.path.abspath(refined_out)
            else:
                extra_message = ""
            out_ppt_file = os.path.abspath(out_ppt_file)
            print(f"\n✅ 转换完成！")
            print(f"📄 输出文件: {out_ppt_file}")
            os.startfile(out_ppt_file)
            messagebox.showinfo("转换成功", f"PDF 已成功转换为 PPT！\n\n文件位置:\n{out_ppt_file}"+extra_message)
        except Exception as e:
            print(f"\n❌ 转换失败: {str(e)}")
            messagebox.showerror("转换失败", f"处理过程中出现错误:\n{str(e)}")
        finally:
            self.start_btn.config(state=tk.NORMAL)

def launch_gui():
    # Enable Windows DPI awareness before creating the Tk root where possible
    try:
        enable_windows_dpi_awareness(None)
    except Exception:
        pass

    root = tk.Tk()
    # After root exists, apply scaling using the helper (this will call tk scaling)
    try:
        enable_windows_dpi_awareness(root)
    except Exception:
        pass

    app = AppGUI(root)
    root.mainloop()

if __name__ == "__main__":
    launch_gui()



# 使用 Spire.Presentation 合并PPT文件，保留原始设计
# 安装: pip install spire.presentation

import os
from spire.presentation import *
from spire.presentation.common import *
from pptx import Presentation as PptxPresentation

def combine_ppt_files_with_spire(source_folder, output_file, png_names=None):
    """
    使用 Spire.Presentation 合并PPT文件，每个PPT只保留第一页，并保留原始设计
    
    Args:
        source_folder: 源PPT文件所在的文件夹路径
        output_file: 输出的合并PPT文件路径
    """
    # 获取所有pptx文件并按字典序排序
    
    ppt_files = sorted([f for f in os.listdir(source_folder) if f.endswith('.pptx')])

    ppt_names = [png_name.replace(".png", ".pptx") for png_name in png_names] if png_names else None
    
    if not ppt_files:
        print("未找到任何PPT文件")
        return
    
    print(f"找到 {len(ppt_files)} 个PPT文件:")
    valid_ppt_files = []
    for idx, file in enumerate(ppt_files, 1):
        print(f"  {idx}. {file}")
        if ppt_names and file in ppt_names:
            valid_ppt_files.append(file)
    if ppt_names:
        ppt_files = valid_ppt_files
        print(f"\n根据提供的PNG名称过滤后，剩余 {len(ppt_files)} 个PPT文件:")
    
    # 创建主演示文稿对象，使用第一个PPT作为基础
    first_ppt_path = os.path.join(source_folder, ppt_files[0])
    main_pres = Presentation()
    main_pres.LoadFromFile(first_ppt_path)
    
    # 删除第一个PPT的多余页面，只保留第一页
    while main_pres.Slides.Count > 1:
        main_pres.Slides.RemoveAt(1)
    
    print(f"  已添加: {ppt_files[0]} (第1页)")
    
    # 遍历剩余的PPT文件
    for ppt_file in ppt_files[1:]:
        file_path = os.path.join(source_folder, ppt_file)
        
        # 加载当前PPT文件
        temp_pres = Presentation()
        temp_pres.LoadFromFile(file_path)
        
        if temp_pres.Slides.Count > 0:
            # 使用 AppendBySlide 方法追加第一页，保留原始设计
            main_pres.Slides.AppendBySlide(temp_pres.Slides[0])
            print(f"  已添加: {ppt_file} (第1页)")
        else:
            print(f"  跳过: {ppt_file} (无幻灯片)")
        
        # 释放临时演示文稿资源
        temp_pres.Dispose()
    
    # 保存合并后的PPT
    main_pres.SaveToFile(output_file, FileFormat.Pptx2016)
    print(f"\n合并完成！输出文件: {output_file}")
    print(f"总共合并了 {main_pres.Slides.Count} 页幻灯片")
    
    # 释放资源
    main_pres.Dispose()

    valid_png_names = [ppt_name.replace(".pptx", ".png") for ppt_name in ppt_files] # 返回真正存在的png names

    return valid_png_names



def clean_ppt(in_ppt_file, out_ppt_file):
    """
    使用 python-pptx 删除PPT中的多余形状
    
    Args:
        ppt_file: 需要清理的PPT文件路径
    """
    ppt = PptxPresentation(in_ppt_file)
    for slide in ppt.slides:
        for shape in list(slide.shapes):
            # 如果shape name 叫做"New shape",删除它
            if shape.name == "New shape":
                sp = slide.shapes._spTree.remove(shape._element)

    ppt.save(out_ppt_file)
    

def combine_ppt(source_folder, out_ppt_file, png_names = None):
    # 确保是字符串路径，因为后面用到了 .replace
    source_folder = str(source_folder)
    out_ppt_file = str(out_ppt_file)
    
    # 方法1: 保留原始设计（推荐）
    output_file1 = out_ppt_file.replace(".pptx", "_combined_original_design.pptx")
    valid_png_names = combine_ppt_files_with_spire(source_folder, output_file1, png_names=png_names)

    clean_ppt(output_file1, out_ppt_file)
    print(f"\n已生成合并的PPT文件: {out_ppt_file}")
    os.remove(output_file1)
    
    return valid_png_names
